@@ -22,31 +22,29 @@
 #include <cstdlib>
 
 #include <Lum/Base/String.h>
+#include <Lum/Base/Util.h>
 
 #include <Lum/OS/Driver.h>
 
-static Lum::Object::Prefs *prefs=new Sweeper::Prefs();
-
 Sweeper::Sweeper()
-: xOff(0),yOff(0),sWidth(0),sHeight(0),bombs(0),oneButtonMode(false),game(NULL),themeIndex(0)
+: sWidth(0),sHeight(0),
+  status(new Lum::Model::ULong()),
+  marks(new Lum::Model::Long()),
+  bombs(0),
+  oneButtonMode(false),
+  game(NULL),
+  themeIndex(0)
 {
-  SetPrefs(::prefs);
+  sMaxWidth=areaWidth;
+  sMaxHeight=areaHeight;
 
-  sMaxWidth=100;
-  sMaxHeight=100;
-
-  status=new Lum::Model::ULong();
   status->Set(waiting);
+  marks->Set(0);
 
   font=Lum::OS::display->GetFont(Lum::OS::Display::fontTypeFixed,
                                  Lum::OS::Display::fontScaleFootnote);
 
-  cellFrame=Lum::OS::display->GetFrame(Lum::OS::Display::buttonFrameIndex);
   cellBackground=Lum::OS::display->GetFill(Lum::OS::Display::buttonBackgroundFillIndex);
-  leftArrow=Lum::OS::display->GetImage(Lum::OS::Display::arrowLeftImageIndex);
-  rightArrow=Lum::OS::display->GetImage(Lum::OS::Display::arrowRightImageIndex);
-  upArrow=Lum::OS::display->GetImage(Lum::OS::Display::arrowUpImageIndex);
-  downArrow=Lum::OS::display->GetImage(Lum::OS::Display::arrowDownImageIndex);
 }
 
 Sweeper::~Sweeper()
@@ -60,13 +58,10 @@ void Sweeper::EvaluateThemeName()
     if (themes[i].name==themeName) {
       themeIndex=i;
 
-      xOff=0;
-      yOff=0;
-
-      if (sWidth>0 && sHeight>0) {
+      if (game!=NULL) {
         delete game;
         game=Lum::OS::driver->CreateBitmap(GetAreaWidth()*GetBoxSize(),
-                                                     GetAreaHeight()*GetBoxSize());
+                                           GetAreaHeight()*GetBoxSize());
         RedrawArea();
 
         Redraw();
@@ -84,7 +79,7 @@ void Sweeper::CalcSize()
   minWidth=10*24;
   minHeight=10*24;
 
-  Lum::Object::CalcSize();
+  Lum::Scrollable::CalcSize();
 }
 
 void Sweeper::DrawCell(size_t row, size_t column)
@@ -95,8 +90,8 @@ void Sweeper::DrawCell(size_t row, size_t column)
   x=row*GetBoxSize();
   y=column*GetBoxSize();
 
-  frameA=std::max(cellFrame->leftBorder,cellFrame->topBorder);
-  frameB=std::max(cellFrame->rightBorder,cellFrame->bottomBorder);
+  frameA=std::max(cellBackground->leftBorder,cellBackground->topBorder);
+  frameB=std::max(cellBackground->rightBorder,cellBackground->bottomBorder);
   frameSize=frameA+frameB;
 
   Lum::OS::FillRef fill=Lum::OS::display->GetFill(Lum::OS::Display::backgroundFillIndex);
@@ -161,21 +156,14 @@ void Sweeper::DrawCell(size_t row, size_t column)
         }
         else {
           cellBackground->Draw(draw,
-                               x+cellFrame->leftBorder,
-                               y+cellFrame->topBorder,
-                               GetBoxSize()-cellFrame->minWidth,
-                               GetBoxSize()-cellFrame->minHeight,
-                               x+cellFrame->leftBorder,
-                               y+cellFrame->topBorder,
-                               GetBoxSize()-cellFrame->minWidth,
-                               GetBoxSize()-cellFrame->minHeight);
+                               x,y,GetBoxSize(),GetBoxSize(),
+                               x,y,GetBoxSize(),GetBoxSize());
           draw->PushForeground(Lum::OS::display->fillColor);
           draw->FillRectangle(x+frameA,
                               y+frameA,
                               GetBoxSize()-frameSize,
                               GetBoxSize()-frameSize);
           draw->PopForeground();
-          cellFrame->Draw(draw,x,y,GetBoxSize(),GetBoxSize());
         }
       }
       // draw the cell as normal hidden cell...
@@ -185,15 +173,8 @@ void Sweeper::DrawCell(size_t row, size_t column)
         }
         else {
           cellBackground->Draw(draw,
-                               x+cellFrame->leftBorder,
-                               y+cellFrame->topBorder,
-                               GetBoxSize()-cellFrame->minWidth,
-                               GetBoxSize()-cellFrame->minHeight,
-                               x+cellFrame->leftBorder,
-                               y+cellFrame->topBorder,
-                               GetBoxSize()-cellFrame->minWidth,
-                               GetBoxSize()-cellFrame->minHeight);
-          cellFrame->Draw(draw,x,y,GetBoxSize(),GetBoxSize());
+                               x,y,GetBoxSize(),GetBoxSize(),
+                               x,y,GetBoxSize(),GetBoxSize());
         }
       }
     }
@@ -261,6 +242,8 @@ void Sweeper::RecalcArea()
   /* automatically open cells which have 0 neighbours */
 
   do {
+    long marks=bombs;
+
     changed=false;
     notFound=false;
 
@@ -296,15 +279,10 @@ void Sweeper::RecalcArea()
           Redraw();
         }
 
-        // still undetected bombs
+        // still unopened areas without bombs
         if (!area[column][row].bomb && !area[column][row].open) {
           notFound=true;
         }
-
-        // still unmarked bombs
-        if (area[column][row].bomb && !area[column][row].marked) {
-          notFound=true;
-        }
       }
     }
 
@@ -326,6 +304,16 @@ void Sweeper::RecalcArea()
       }
     }
 
+    for (size_t column=0; column<sWidth; column++) {
+      for (size_t row=0; row<sHeight; row++) {
+        if (area[column][row].marked) {
+          marks--;
+        }
+      }
+    }
+
+    this->marks->Set(marks);
+
     if ((status->Get()==playing) && !notFound) {
       timer.Pause();
       status->Set(won);
@@ -343,63 +331,6 @@ Lum::Images::Image* Sweeper::GetImage(Image image) const
   return themes[themeIndex].images[image];
 }
 
-bool Sweeper::IsAreaCompletelyVisible() const
-{
-  return GetAreaWidth()<=width/GetBoxSize() && GetAreaHeight()<=height/GetBoxSize();
-}
-
-bool Sweeper::CanScrollLeft() const
-{
-  return xOff>0;
-}
-
-bool Sweeper::CanScrollUp() const
-{
-  return yOff>0;
-}
-
-bool Sweeper::CanScrollRight() const
-{
-  return xOff+(width/GetBoxSize()-2)<GetAreaWidth();
-}
-
-bool Sweeper::CanScrollDown() const
-{
-  return yOff+(height/GetBoxSize()-2)<GetAreaHeight();
-}
-
-void Sweeper::ScrollLeft()
-{
-  assert(xOff>0);
-
-  xOff--;
-  Redraw();
-}
-
-void Sweeper::ScrollUp()
-{
-  assert(yOff>0);
-
-  yOff--;
-  Redraw();
-}
-
-void Sweeper::ScrollRight()
-{
-  //assert();
-
-  xOff++;
-  Redraw();
-}
-
-void Sweeper::ScrollDown()
-{
-  //assert();
-
-  yOff++;
-  Redraw();
-}
-
 void Sweeper::RedrawArea()
 {
   for (size_t x=0; x<sWidth; x++) {
@@ -445,11 +376,21 @@ void Sweeper::ToggleMark(size_t x, size_t y)
   }
 }
 
+void Sweeper::Layout()
+{
+  if (game!=NULL) {
+    hAdjustment->SetDimension(width,game->GetWidth());
+    vAdjustment->SetDimension(height,game->GetHeight());
+  }
+
+  Scrollable::Layout();
+}
+
 void Sweeper::Draw(int x, int y, size_t w, size_t h)
 {
-  Object::Draw(x,y,w,h);
+  Scrollable::Draw(x,y,w,h);
 
-  if (!Intersect(x,y,w,h)) {
+  if (!OIntersect(x,y,w,h)) {
     return;
   }
 
@@ -457,79 +398,16 @@ void Sweeper::Draw(int x, int y, size_t w, size_t h)
 
   DrawBackground(x,y,w,h);
 
-  if (sWidth==0 || sHeight==0) {
+  if (game==NULL) {
     return;
   }
 
   Lum::OS::DrawInfo *draw=GetDrawInfo();
 
-  if (IsAreaCompletelyVisible()) {
-    draw->CopyFromBitmap(game,
-                         0,0,GetAreaWidth()*GetBoxSize(),GetAreaHeight()*GetBoxSize(),
-                         this->x,this->y);
-  }
-  else {
-    draw->CopyFromBitmap(game,
-                         xOff*GetBoxSize(),yOff*GetBoxSize(),
-                         width-2*GetBoxSize(),
-                         height-2*GetBoxSize(),
-                         this->x+GetBoxSize(),this->y+GetBoxSize());
-
-    if (CanScrollUp()) {
-      if (GetImage(upImage)!=NULL) {
-        GetImage(upImage)->Draw(draw,
-                                this->x+(width-GetBoxSize())/2,
-                                this->y);
-      }
-      else {
-        upArrow->Draw(draw,
-                      this->x+(width-GetBoxSize())/2,this->y,
-                      GetBoxSize(),GetBoxSize());
-      }
-    }
-
-    // Left
-    if (CanScrollLeft()) {
-      if (GetImage(leftImage)!=NULL) {
-        GetImage(leftImage)->Draw(draw,
-                                  this->x,
-                                  this->y+(height-GetBoxSize())/2);
-      }
-      else {
-         leftArrow->Draw(draw,
-                         this->x,this->y+(height-GetBoxSize())/2,
-                         GetBoxSize(),GetBoxSize());
-      }
-    }
-
-    // Down
-    if (CanScrollDown()) {
-      if (GetImage(downImage)!=NULL) {
-        GetImage(downImage)->Draw(draw,
-                                  this->x+(width-GetBoxSize())/2,
-                                  this->y+height-GetBoxSize());
-      }
-      else {
-         downArrow->Draw(draw,
-                         this->x+(width-GetBoxSize())/2,this->y+height-GetBoxSize(),
-                         GetBoxSize(),GetBoxSize());
-      }
-    }
-
-    // Right
-    if (CanScrollRight()) {
-      if (GetImage(rightImage)!=NULL) {
-        GetImage(rightImage)->Draw(draw,
-                                   this->x+width-GetBoxSize(),
-                                   this->y+(height-GetBoxSize())/2);
-      }
-      else {
-         rightArrow->Draw(draw,
-                          this->x+width-GetBoxSize(),this->y+(height-GetBoxSize())/2,
-                          GetBoxSize(),GetBoxSize());
-      }
-    }
-  }
+  draw->CopyFromBitmap(game,
+                       hAdjustment->GetTop()-1,vAdjustment->GetTop()-1,
+                       this->width,this->height,
+                       this->x,this->y);
 }
 
 bool Sweeper::HandleMouseEvent(const Lum::OS::MouseEvent& event)
@@ -549,31 +427,8 @@ bool Sweeper::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
     size_t x,y;
 
-    if (!IsAreaCompletelyVisible()) {
-      if ((size_t)event.x<this->x+GetBoxSize() && CanScrollLeft()) {
-        ScrollLeft();
-        return true;
-      }
-      if ((size_t)event.x>=this->x+width-GetBoxSize() && CanScrollRight()) {
-        ScrollRight();
-        return true;
-      }
-      if ((size_t)event.y<this->y+GetBoxSize() && CanScrollUp()) {
-        ScrollUp();
-        return true;
-      }
-      if ((size_t)event.y>=this->y+height-GetBoxSize() && CanScrollDown()) {
-        ScrollDown();
-        return true;
-      }
-
-      x=(event.x-this->x)/GetBoxSize()-1+xOff;
-      y=(event.y-this->y)/GetBoxSize()-1+yOff;
-    }
-    else {
-      x=(event.x-this->x)/GetBoxSize();
-      y=(event.y-this->y)/GetBoxSize();
-    }
+    x=(event.x-this->x+hAdjustment->GetTop()-1)/GetBoxSize();
+    y=(event.y-this->y+vAdjustment->GetTop()-1)/GetBoxSize();
 
     if (oneButtonMode) {
       if ((size_t)std::abs(mbdx-event.x)>=GetBoxSize()/3 || (size_t)std::abs(mbdy-event.y)>=GetBoxSize()/3) {
@@ -599,27 +454,8 @@ bool Sweeper::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
     size_t x,y;
 
-    if (!IsAreaCompletelyVisible()) {
-      if ((size_t)event.x<this->x+GetBoxSize() && CanScrollLeft()) {
-        return true;
-      }
-      if ((size_t)event.x>=this->x+width-GetBoxSize() && CanScrollRight()) {
-        return true;
-      }
-      if ((size_t)event.y<this->y+GetBoxSize() && CanScrollUp()) {
-        return true;
-      }
-      if ((size_t)event.y>=this->y+height-GetBoxSize() && CanScrollDown()) {
-        return true;
-      }
-
-      x=(event.x-this->x)/GetBoxSize()-1+xOff;
-      y=(event.y-this->y)/GetBoxSize()-1+yOff;
-    }
-    else {
-      x=(event.x-this->x)/GetBoxSize();
-      y=(event.y-this->y)/GetBoxSize();
-    }
+    x=(event.x-this->x+hAdjustment->GetTop()-1)/GetBoxSize();
+    y=(event.y-this->y+vAdjustment->GetTop()-1)/GetBoxSize();
 
     ToggleMark(x,y);
 
@@ -631,11 +467,6 @@ bool Sweeper::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
 bool Sweeper::SetSize(size_t width, size_t height, size_t bombs)
 {
-  if (width==sWidth && height==sHeight && bombs==this->bombs) {
-    // Don't erase game zone, if nothing has changed.
-    return true;
-  }
-
   if (width>sMaxWidth || height>sMaxHeight || bombs>width*height) {
     return false;
   }
@@ -651,6 +482,8 @@ bool Sweeper::SetSize(size_t width, size_t height, size_t bombs)
   Run();
 
   RedrawArea();
+  hAdjustment->SetDimension(this->width,game->GetWidth());
+  vAdjustment->SetDimension(this->height,game->GetHeight());
 
   Redraw();
 
@@ -677,11 +510,21 @@ size_t Sweeper::GetMines() const
   return bombs;
 }
 
+size_t Sweeper::GetMinesPercent() const
+{
+  return Lum::Base::RoundDiv(bombs*100,sWidth*sHeight);
+}
+
 Lum::Model::ULong* Sweeper::GetStatusModel() const
 {
   return status.Get();
 }
 
+Lum::Model::Long* Sweeper::GetMarksModel() const
+{
+  return marks.Get();
+}
+
 void Sweeper::Run()
 {
   size_t cells,bombs;
@@ -718,12 +561,10 @@ void Sweeper::Run()
   }
 
   status->Set(playing);
+  marks->Set(this->bombs);
 
   RedrawArea();
 
-  xOff=0;
-  yOff=0;
-
   Redraw();
 
   timer.Reset();
@@ -746,3 +587,13 @@ void Sweeper::SetStatusToWaiting()
     Redraw();
   }
 }
+
+void Sweeper::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+{
+  if (model==hAdjustment->GetTopModel() || model==vAdjustment->GetTopModel()) {
+    Redraw();
+  }
+
+  Lum::Scrollable::Resync(model,msg);
+}
+
