@@ -33,7 +33,7 @@
 
 #include "Configuration.h"
 
-class Sweeper : public Lum::Object
+class Sweeper : public Lum::Scrollable
 {
 public:
   static const int areaWidth=100;
@@ -58,17 +58,14 @@ private:
   };
 
 private:
-  size_t               xOff; // Top left visible coordinate of cell (in cell index)
-  size_t               yOff;
   size_t               sWidth;
   size_t               sHeight;
   size_t               sMaxWidth;
   size_t               sMaxHeight;
-  Lum::OS::FrameRef    cellFrame;
   Lum::OS::FillRef     cellBackground;
   Lum::OS::FontRef     font;
-  Lum::OS::ImageRef    upArrow,downArrow,leftArrow,rightArrow;
   Lum::Model::ULongRef status;
+  Lum::Model::LongRef  marks;
   size_t               bombs;
   Cell                 area[areaWidth][areaHeight];
   Lum::Base::Timer     timer;
@@ -83,18 +80,6 @@ private:
   size_t GetBoxSize() const;
   Lum::Images::Image* GetImage(Image image) const;
 
-  bool IsAreaCompletelyVisible() const;
-
-  bool CanScrollLeft() const;
-  bool CanScrollUp() const;
-  bool CanScrollRight() const;
-  bool CanScrollDown() const;
-
-  void ScrollLeft();
-  void ScrollUp();
-  void ScrollRight();
-  void ScrollDown();
-
   void DrawCell(size_t row, size_t column);
   void RedrawArea();
 
@@ -114,6 +99,7 @@ public:
 
   void CalcSize();
 
+  void Layout();
   void Draw(int x, int y, size_t w, size_t h);
 
   bool HandleMouseEvent(const Lum::OS::MouseEvent& event);
@@ -124,14 +110,18 @@ public:
   size_t GetAreaWidth() const;
   size_t GetAreaHeight() const;
   size_t GetMines() const;
+  size_t GetMinesPercent() const;
 
   Lum::Model::ULong* GetStatusModel() const;
+  Lum::Model::Long* GetMarksModel() const;
 
   void Run();
 
   Status GetStatus() const;
   time_t GetElapsedTime() const;
   void   SetStatusToWaiting();
+
+  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg);
 };
 
 typedef Sweeper* SweeperPtr;
