@@ -23,6 +23,7 @@
 
 #include <Lum/Base/Object.h>
 #include <Lum/Base/String.h>
+#include <Lum/Base/Util.h>
 
 #include <Lum/Dlg/About.h>
 #include <Lum/Dlg/Msg.h>
@@ -49,6 +50,7 @@
 #include <Lum/Panel.h>
 #include <Lum/Space.h>
 #include <Lum/StatusLine.h>
+#include <Lum/View.h>
 #include <Lum/WindowGroup.h>
 
 #include "config.h"
@@ -105,7 +107,6 @@ public:
     table=new Lum::Table();
     table->SetFlex(true,true);
     table->SetMinWidth(Lum::Base::Size::stdCharWidth,30);
-    table->RequestFocus();
     table->SetShowHeader(true);
     table->GetTableView()->SetAutoFitColumns(true);
     table->GetTableView()->SetAutoVSize(true);
@@ -121,7 +122,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeDefault);
     button->SetText(L"_Close");
@@ -195,7 +195,6 @@ public:
         string=new Lum::String();
         string->SetFlex(true,false);
         string->SetWidth(Lum::Base::Size::stdCharWidth,10);
-        string->RequestFocus();
         string->SetModel(name);
         grid->SetObject(1,x,string);
       }
@@ -234,7 +233,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(closeAction);
     button->SetType(Lum::Button::typeDefault);
     button->SetText(L"_Close");
@@ -323,7 +321,6 @@ public:
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->RequestFocus();
     table->SetShowHeader(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
@@ -339,7 +336,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(okAction);
     button->SetType(Lum::Button::typeCommit);
     button->SetText(L"_Ok");
@@ -347,7 +343,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeCancel);
     button->SetText(L"_Cancel");
@@ -411,7 +406,7 @@ public:
     themeAction(new Lum::Model::Action()),
     horiz(new Lum::Model::Int(sweeper->GetAreaWidth())),
     vert(new Lum::Model::Int(sweeper->GetAreaHeight())),
-    mines(new Lum::Model::Int((sweeper->GetMines()*100)/(sweeper->GetAreaWidth()*sweeper->GetAreaHeight()))),
+    mines(new Lum::Model::Int(sweeper->GetMinesPercent())),
     oneButtonMode(new Lum::Model::Boolean(GetOneButtonMode())),
     themeName(new Lum::Model::String())
   {
@@ -455,21 +450,18 @@ public:
 
     slider=new Lum::HSlider();
     slider->SetFlex(true,false);
-    slider->RequestFocus();
     slider->SetScale(true);
     slider->SetModel(horiz);
     label->AddLabel(L"Horizontal:",slider);
 
     slider=new Lum::HSlider();
     slider->SetFlex(true,false);
-    slider->RequestFocus();
     slider->SetScale(true);
     slider->SetModel(vert);
     label->AddLabel(L"Vertical:",slider);
 
     slider=new Lum::HSlider();
     slider->SetFlex(true,false);
-    slider->RequestFocus();
     slider->SetScale(true);
     slider->SetModel(mines);
     label->AddLabel(L"Mines (in %):",slider);
@@ -481,21 +473,18 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(smallAction);
     button->SetText(L"_Small");
     array->Add(button);
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(mediumAction);
     button->SetText(L"_Medium");
     array->Add(button);
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(bigAction);
     button->SetText(L"_Big");
     array->Add(button);
@@ -503,7 +492,6 @@ public:
     label->AddLabel(L"Presets:",array);
 
     boolean=new Lum::Boolean();
-    boolean->RequestFocus();
     boolean->SetModel(oneButtonMode);
 
     label->AddLabel(L"One mouse button mode:",boolean);
@@ -518,7 +506,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(false,true);
-    button->RequestFocus();
     button->SetModel(themeAction);
     button->SetText(L"...");
     hPanel->Add(button);
@@ -535,7 +522,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeDefault);
     button->SetText(L"_Close");
@@ -554,18 +540,17 @@ public:
     if (model==smallAction && smallAction->IsFinished()) {
       horiz->Set(8);
       vert->Set(8);
-      mines->Set((10*100)/(8*8));
+      mines->Set(Lum::Base::RoundDiv(10*100,8*8));
     }
     else if (model==mediumAction && mediumAction->IsFinished()) {
       horiz->Set(16);
       vert->Set(16);
-      mines->Set((40*100)/(16*16));
+      mines->Set(Lum::Base::RoundDiv(40*100,16*16));
     }
     else if (model==bigAction && bigAction->IsFinished()) {
       horiz->Set(30);
       vert->Set(16);
-      mines->Set(99);
-      mines->Set((99*100)/(30*16));
+      mines->Set(Lum::Base::RoundDiv(99*100,30*16));
     }
     else if (model==themeAction && themeAction->IsFinished()) {
       SelectTheme *dialog=new SelectTheme();
@@ -601,6 +586,7 @@ private:
   Lum::Model::ActionRef aboutAction;
   Lum::Model::ActionRef timerAction;
 
+  Lum::Text             *marks;
   Lum::Text             *time;
 
 public:
@@ -642,9 +628,10 @@ public:
     Lum::MenuStrip   *strip;
     Lum::Menu        *menu;
     Lum::Panel       *vPanel,*hPanel;
+    Lum::View        *view;
     Lum::WindowGroup *wGroup;
-    size_t           areaWidth,areaHeight,areaBombs,digitWidth,widestDigit;
-    std::wstring     timeFormat;
+    size_t           digitWidth,widestDigit;
+    std::wstring     timeFormat,marksFormat;
     Lum::OS::FontRef font;
 
     wGroup=new Lum::WindowGroup();
@@ -657,14 +644,13 @@ public:
     hPanel->SetFlex(true,false);
 
     button=new Lum::Button();
-    button->RequestFocus();
     button->SetText(L"_New game");
     button->SetModel(newGameAction);
     hPanel->Add(button);
 
     hPanel->Add(new Lum::HSpace(Lum::Space::sizeNormal,true));
 
-    font=Lum::OS::display->GetFont(Lum::OS::Display::fontTypeFixed,200);
+    font=Lum::OS::display->GetFont(Lum::OS::Display::fontTypeFixed,210);
 
     digitWidth=0;
     widestDigit=0;
@@ -679,15 +665,32 @@ public:
       }
     }
 
-    for (size_t x=0; x<4; x++) {
+    marksFormat=L"-";
+    for (size_t x=1; x<=3; x++) {
+      marksFormat.append(Lum::Base::NumberToWString(widestDigit));
+    }
+
+    marks=new Lum::Text();
+    marks->SetFont(font);
+    marks->SetStyle(Lum::OS::Font::bold);
+    marks->SetAlignment(Lum::Text::right);
+    marks->SetMinWidth(Lum::Base::Size::modePixel,font->StringWidth(marksFormat));
+    marks->SetText(L"0");
+
+    hPanel->Add(marks);
+
+    hPanel->Add(new Lum::HSpace(Lum::Space::sizeNormal,true));
+
+    for (size_t x=1; x<=4; x++) {
       timeFormat.append(Lum::Base::NumberToWString(widestDigit));
     }
 
     time=new Lum::Text();
     time->SetFont(font);
     time->SetStyle(Lum::OS::Font::bold);
-    time->SetAlignment(Lum::Text::right);
-    time->SetText(timeFormat);
+    time->SetAlignment(Lum::Text::centered);
+    time->SetMinWidth(Lum::Base::Size::modePixel,font->StringWidth(timeFormat));
+    time->SetText(L"0000");
 
     hPanel->Add(time);
 
@@ -695,11 +698,16 @@ public:
 
     vPanel->Add(new Lum::VSpace());
 
+    view=new Lum::View();
+    view->SetFlex(true,true);
+
     sweeper=new Sweeper();
     sweeper->SetFlex(true,true);
-    GetCurrentGame(areaWidth,areaHeight,areaBombs);
     AttachModel(sweeper->GetStatusModel());
-    vPanel->Add(sweeper);
+    AttachModel(sweeper->GetMarksModel());
+
+    view->SetObject(sweeper);
+    vPanel->Add(view);
 
     strip=new Lum::MenuStrip();
 
@@ -759,8 +767,9 @@ public:
       settings->EventLoop();
       settings->Close();
 
-      if (sweeper->SetSize(settings->horiz->Get(),settings->vert->Get(),
-                           (settings->horiz->Get()*settings->vert->Get()*settings->mines->Get())/100)) {
+      if (sweeper->SetSize(settings->horiz->Get(),
+                           settings->vert->Get(),
+                           Lum::Base::RoundDiv(settings->horiz->Get()*settings->vert->Get()*settings->mines->Get(),100))) {
         SetCurrentGame(sweeper->GetAreaWidth(),
                        sweeper->GetAreaHeight(),
                        sweeper->GetMines());
@@ -928,6 +937,9 @@ public:
       }
       Lum::OS::display->AddTimer(1,0,timerAction);
     }
+    else if (sweeper!=NULL && model==sweeper->GetMarksModel()) {
+      this->marks->SetText(Lum::Base::NumberToWString(sweeper->GetMarksModel()->Get()));
+    }
     else if (model==GetWindow()->GetMapedAction() && GetWindow()->GetMapedAction()->IsFinished()) {
       Lum::OS::display->AddTimer(1,0,timerAction);
     }
